/***************************************************************************
                           kdetvimage.cpp
                           --------------
    begin                : Sat Jun 12 2004
    copyright            : (C) 2004 by Dirk Ziegelmeier
                           (C) 2002 by George Staikos
    email                : dziegel@gmx.de
 ***************************************************************************/

/*
 * This library is free software; you can redistribute it and/or
 * modify it under the terms of the GNU Library General Public
 * License as published by the Free Software Foundation; either
 * version 2 of the License, or (at your option) any later version.
 *
 * This library is distributed in the hope that it will be useful,
 * but WITHOUT ANY WARRANTY; without even the implied warranty of
 * MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the GNU
 * Library General Public License for more details.
 *
 * You should have received a copy of the GNU Library General Public License
 * along with this library; see the file COPYING.LIB.  If not, write to
 * the Free Software Foundation, Inc., 59 Temple Place - Suite 330,
 * Boston, MA 02111-1307, USA.
 */

#include <qimage.h>
#include <kdebug.h>

#include "kdetvimage.h"
#include "kdetvimagepool.h"

KdetvImage::KdetvImage()
    : _size(QSize()),
      _format(FORMAT_NONE),
      _type(TYPE_PROGRESSIVE),
      _stride(0),
      _bufSize(0),
      _buf(NULL),
      _bufowner(false)
{
}

KdetvImage::~KdetvImage()
{
    deleteBuffer();
}

QSize KdetvImage::setSize(QSize size)
{
    _size = size;
    return _size;
}

KdetvImage::ImageFormat KdetvImage::setFormat(ImageFormat format)
{
    _format = format;
    return _format;
}

void KdetvImage::setStride(unsigned int stride)
{
    _stride = stride;
}

void KdetvImage::setType(ImageType type)
{
    _type = type;
}

void KdetvImage::setBuffer(unsigned char* buf, unsigned int size, bool bufowner)
{
    deleteBuffer();

    _buf = buf;
    _bufSize = size;
    _bufowner = bufowner;
}

void KdetvImage::createBuffer(unsigned int size)
{
    deleteBuffer();

    _buf = new unsigned char[size];
    _bufSize = size;
    _bufowner = true;
}

void KdetvImage::deleteBuffer()
{
    if(_bufowner) {
        delete[] _buf;
    }
    _buf = 0;
    _bufSize = 0;
    _bufowner = false;
}

bool KdetvImage::toQImage(QImage& img)
{
	if (!_size.isValid()) {
		return false;
	}
    
	switch (_format) {
	case FORMAT_RGB32:
		img.create(_size.width(), _size.height(), 32);
		memcpy(img.bits(), buffer(), _size.width()*_size.height()*bytesPerPixel());
		break;
	case FORMAT_RGB24:
		{
            img.create(_size.width(), _size.height(), 32);
            uchar *bits = img.bits();
			for (int i = 0; i < _size.width()*_size.height(); i++) {
				bits[4*i  ] = buffer()[3*i  ];
				bits[4*i+1] = buffer()[3*i+1];
				bits[4*i+2] = buffer()[3*i+2];
				bits[4*i+3] = 0;
			}
		}
		break;
	case FORMAT_RGB565:
		kdDebug() <<  "KdetvImage(RGB565)->QImage conversion: "
                  << _size.width() << "x" << _size.height() << " " << bytesPerPixel() << "Bpp" << endl;
		img.create(_size.width(), _size.height(), 16);
		memcpy(img.bits(), buffer(), _size.width()*_size.height()*bytesPerPixel());
		break;
	case FORMAT_RGB555:
		kdDebug() <<  "KdetvImage(RGB555)->QImage conversion: "
                  << _size.width() << "x" << _size.height() << " " << bytesPerPixel() << "Bpp" << endl;
		img.create(_size.width(), _size.height(), 15);
		memcpy(img.bits(), buffer(), _size.width()*_size.height()*bytesPerPixel());
		break;
	default:
		// unknown or unhandled format
        return false;
        break;
	}

    return true;
}

const char* KdetvImage::toString(KdetvImage::ImageFormat fmt)
{
    switch(fmt) {
    case KdetvImage::FORMAT_YUV422:
        return "KdetvImageFormat::FORMAT_YUV422";
        break;
    case KdetvImage::FORMAT_YUYV:
        return "KdetvImageFormat::FORMAT_YUYV";
        break;
    case KdetvImage::FORMAT_UYVY:
        return "KdetvImageFormat::FORMAT_UYVY";
        break;
    case KdetvImage::FORMAT_YUV420:
        return "KdetvImageFormat::FORMAT_YUV420";
        break;
    case KdetvImage::FORMAT_YUV411:
        return "KdetvImageFormat::FORMAT_YUV411";
        break;
    case KdetvImage::FORMAT_RGB24:
        return "KdetvImageFormat::FORMAT_RGB24";
        break;
    case KdetvImage::FORMAT_RGB32:
        return "KdetvImageFormat::FORMAT_RGB32";
        break;
    case KdetvImage::FORMAT_RGB555:
        return "KdetvImageFormat::FORMAT_RGB555";
        break;
    case KdetvImage::FORMAT_RGB565:
        return "KdetvImageFormat::FORMAT_RGB565";
        break;
    case KdetvImage::FORMAT_YUV422P:
        return "KdetvImageFormat::FORMAT_YUV422P";
        break;
    case KdetvImage::FORMAT_YUV411P:
        return "KdetvImageFormat::FORMAT_YUV411P";
        break;
    case KdetvImage::FORMAT_YUV420P:
        return "KdetvImageFormat::FORMAT_YUV420P";
        break;
    case KdetvImage::FORMAT_YUV410P:
        return "KdetvImageFormat::FORMAT_YUV410P";
        break;
    case KdetvImage::FORMAT_GREY:
        return "KdetvImageFormat::FORMAT_GREY";
        break;
    case KdetvImage::FORMAT_HI240:
        return "KdetvImageFormat::FORMAT_HI240";
        break;
    default:
        return "KdetvImageFormat: invalid or multiple formats specified";
        break;
    }
}

unsigned int KdetvImage::bytesppForFormat(ImageFormat fmt)
{
    switch (fmt) {
    case FORMAT_RGB32:
    case FORMAT_RGB24:
        return 4;
        
    case FORMAT_RGB555:
    case FORMAT_RGB565:
    case FORMAT_YUV422:
    case FORMAT_YUYV:
    case FORMAT_UYVY:
    case FORMAT_YUV420:
    case FORMAT_YUV411:
    case FORMAT_YUV422P:
    case FORMAT_YUV411P:
    case FORMAT_YUV420P:
    case FORMAT_YUV410P:
        return 2;
        
    case FORMAT_GREY:
        return 1;
        
    case FORMAT_HI240:
    default:
        // unknown format
        return 0;
    }
}

// ------------------------------------------------------------------------------------

KdetvSharedImage::KdetvSharedImage()
    : KdetvImage(),
      _refCount(1)
{
}

KdetvSharedImage::~KdetvSharedImage()
{
}

KdetvSharedImage& KdetvSharedImage::newRef()
{
    _refCount++;
    return *this;
}

void KdetvSharedImage::deleteRef()
{
    _refCount--;
    if(_refCount == 0) {
        delete this;
    }
}

// ------------------------------------------------------------------------------------

KdetvPooledImage::KdetvPooledImage(KdetvImagePool* pool)
    : KdetvSharedImage(),
      _pool(pool)
{
}

KdetvPooledImage::~KdetvPooledImage()
{
}

void* KdetvPooledImage::operator new(size_t, KdetvImagePool* pool)
{
    return pool->getImageStorage();
}

void KdetvPooledImage::operator delete(void* instance)
{
    ((KdetvPooledImage*)instance)->_pool->putImage((KdetvPooledImage*)instance);
}
